# Conditionals

Conditionals allow you to determine whether or not to execute a specific action based on the evaluation of an expression.

## if statement

### Forms

#### if 

```bash
if expression; then

    code if 'expression' is true.

fi
```

#### if..else statement

```bash
if expression; then

    code if 'expression' is true.
else
    code if 'expression' is false.
fi
```

#### if..elif..else statement

```bash
if expression1; then
    code if 'expression1' is true.
elif expression2; then
    code if 'expression2' is true.
else
    code if 'expression1' AND  'expression2' are false.
fi
```

### Expression

In Bash, there are two ways to use conditional expressions: `[ expression ]` and `[[ expression ]]`. While they may seem similar, there are some important differences between them.

#### [ expression ] 

Legacy: This is the older, POSIX-compliant test command that works in many shells, not just Bash.  
It has limited functionality. It lacks advanced features found in `[[ expression ]]`.  
If you need your script to work across different shells (e.g., Dash, POSIX shell), you should stick with [ ].

#### [[ expression ]]

You should prioritize the use of this form in bash. It’s more flexible, safer, and easier to read and write.

* String matching: It supports pattern matching with == and != (using *, ?, etc.).
* Logical operators: You can use && (AND) and || (OR) operators, making it easier to read and use.
* Regular Expressions: It supports regex with =~ for regular expression matching.

### Test and comparison operators

#### File tests

```bash
-f file1  # True if file file1 exists
-d dir1   # True if directory dir1 exists

file1 -nf file2 # True if file1 is newer than file2, or if file1exists and file2 does not
file1 -ot file2 # True if file1 is older than file2, or if file2 exists and file1 does not
```

#### String 

##### String comparison

```bash
string1 == string2  # True if string1 and string are equal
string1 != string2  # True if string1 and string are different
string1 =~ regularExpression # True if string1 match the pattern of the regular expression
```

##### String test

```bash
-z string # True if string is empty
-n string # True if string is non-empty
```

#### Integer comparison 

##### [[ int1 operator int2 ]]

```bash
int1 -eq int2 # True if int1 is equal to int2
int1 -ne int2 # True if int1 is not equal to int2
int1 -lt int2 # True if int1 is less than to int2
int1 -le int2 # True if int1 is less than or equal to int2
int1 -gt int2 # True if int1 is greater than to int2
int1 -ge int2 # True if int1 is greater than or equal to int2
```

##### (( int1 operator int2 ))

Double parenthesis is used specifically for numeric operations.

```bash
int1 == int2 # True if int1 is equal to int2
int1 != int2 # True if int1 is not equal to int2
int1 < int2 # True if int1 is less than to int2
int1 <= int2 # True if int1 is less than or equal to int2
int1 > int2 # True if int1 is greater than to int2
int1 >= int2 # True if int1 is greater than or equal to int2
```

##### [[ ... ]] vs (( ... ))

| Feature | [[ ... ]] | (( ... )) |
|----------|----------|----------|
| Used for | Strings & Integers|Only Integers|
|Operators|-eq, -ne, -lt, -le, -gt, -ge|==, !=, <, <=, >, >=|
|Variable $ needed|Yes|No|
|Supports Arithmetic|No|Yes|
|Faster for math?|No|Yes|

### Logical operators

Logical operators in Bash are used to combine multiple conditions.

#### NOT (!)

`!` negates a condition. Example:

```bash
if [ ! -d myfolder ]; then
    echo "Directory does not exist"
fi
```

#### AND (&&)

Check if both conditions are true.

```bash
if [[ -f file.txt && -r file.txt ]]; then
    echo "File exists and is readable"
fi
```

#### OR (||)

Check if one of the conditions is true.

```bash
if [[ ! -f file.txt || ! -r file.txt ]]; then
    echo "File does not exist or is not readable."
fi
```

In this example we used the negation `!` and the `||` logical operators.

## case statement

The Bash case statement is a powerful tool for simplifying complex conditionals, especially when handling multiple choices. It provides a cleaner and more readable alternative to nested if statements, making scripts easier to maintain. A common use case is processing arguments passed to a script.

```bash
case $variable in

    pattern_1)
        command1
        ;;
    pattern_2|pattern_3|pattern_4)
        command2
        ;;
    pattern_N)
        commandN
        ;;
    *)
        commands
        ;;
esac
```

A common practice is to use the wildcard asterisk (*) as the final pattern in a case statement, ensuring a default case that matches any input.


!!! Success "Quick recap"
    In this section we've learned:

    - The if, elif and else statements
    - difference between [ ... ] [[ ... ]] (( ... )) expression structure
    - File test operator
    - String and integer comparison operators
    - Logical operators
    - The case statement